@@ -183,6 +183,41 @@ DEFAULT_ERROR_MSG_INV="Running {{'it' if parent['part'] else 'the higlighted exp
 DEFAULT_UNDEFINED_NAME_MSG="Running {{'it' if parent['part'] else 'the higlighted expression'}} should define a variable `{{name}}` without errors, but it doesn't."
 DEFAULT_INCORRECT_NAME_MSG="Are you sure you assigned the correct value to `{{name}}`?"
 DEFAULT_INCORRECT_EXPR_CODE_MSG="Running the expression `{{expr_code}}` didn't generate the expected result."
+
+args_string = """
+
+    Args:
+        incorrect_msg (str): feedback message if the {0} of the expression in the solution
+          doesn't match the one of the student. This feedback message will be expanded if it is used
+          in the context of another check function, like ``check_if_else``.
+        error_msg (str): feedback message if there was an error when running the targeted student code.
+          Note that when testing for an error, this message is displayed when none is raised.
+        undefined_msg (str): feedback message if the ``name`` argument is defined, but a variable
+          with that name doesn't exist after running the targeted student code.
+        extra_env (dict): set variables to the extra environment. They will update the student and solution environment in
+          the active state before the student/solution code in the active state is ran. This argument should contain a
+          dictionary with the keys the names of the variables you want to set, and the values are the values of these variables.
+          You can also use ``set_env()`` for this.
+        context_vals (list): set variables which are bound in a ``for`` loop to certain values.
+          This argument is only useful when checking a for loop (or list comprehensions).
+          It contains a list with the values of the bound variables.
+          You can also use ``set_context()`` for this.
+        pre_code (str): the code in string form that should be executed before the expression is executed.
+          This is the ideal place to set a random seed, for example.
+        expr_code (str): if this argument is set, the expression in the student/solution code will not
+          be ran. Instead, the given piece of code will be ran in the student as well as the solution environment
+          and the result will be compared.
+        name (str): If this is specified, the {0} of running this expression after running the focused expression
+          is returned, instead of the {0} of the focussed expression in itself. This is typically used to inspect the
+          {0} of an object after executing the body of e.g. a ``for`` loop.
+        copy (bool): whether to try to deep copy objects in the environment, such as lists, that could
+          accidentally be mutated. Disable to speed up SCTs. Disabling may lead to cryptic mutation issues.
+        func: custom binary function of form f(stu_result, sol_result), for equality testing.
+        override: If specified, this avoids the execution of the targeted code in the solution process. Instead, it
+          will compare the {0} of the expression in the student process with the value specified in ``override``.
+          Typically used in a ``SingleProcessExercise`` or if you want to allow for different solutions other than
+          the one coded up in the solution.
+    """
 def has_expr(incorrect_msg=None,
              error_msg=None,
              undefined_msg=None,
@@ -282,43 +317,6 @@ def has_expr(incorrect_msg=None,
 
     return state
 
-
-
-args_string = """
-
-    Args:
-        incorrect_msg (str): feedback message if the {0} of the expression in the solution
-          doesn't match the one of the student. This feedback message will be expanded if it is used
-          in the context of another check function, like ``check_if_else``.
-        error_msg (str): feedback message if there was an error when running the targeted student code.
-          Note that when testing for an error, this message is displayed when none is raised.
-        undefined_msg (str): feedback message if the ``name`` argument is defined, but a variable
-          with that name doesn't exist after running the targeted student code.
-        extra_env (dict): set variables to the extra environment. They will update the student and solution environment in
-          the active state before the student/solution code in the active state is ran. This argument should contain a
-          dictionary with the keys the names of the variables you want to set, and the values are the values of these variables.
-          You can also use ``set_env()`` for this.
-        context_vals (list): set variables which are bound in a ``for`` loop to certain values.
-          This argument is only useful when checking a for loop (or list comprehensions).
-          It contains a list with the values of the bound variables.
-          You can also use ``set_context()`` for this.
-        pre_code (str): the code in string form that should be executed before the expression is executed.
-          This is the ideal place to set a random seed, for example.
-        expr_code (str): if this argument is set, the expression in the student/solution code will not
-          be ran. Instead, the given piece of code will be ran in the student as well as the solution environment
-          and the result will be compared.
-        name (str): If this is specified, the {0} of running this expression after running the focused expression
-          is returned, instead of the {0} of the focussed expression in itself. This is typically used to inspect the
-          {0} of an object after executing the body of e.g. a ``for`` loop.
-        copy (bool): whether to try to deep copy objects in the environment, such as lists, that could
-          accidentally be mutated. Disable to speed up SCTs. Disabling may lead to cryptic mutation issues.
-        func: custom binary function of form f(stu_result, sol_result), for equality testing.
-        override: If specified, this avoids the execution of the targeted code in the solution process. Instead, it
-          will compare the {0} of the expression in the student process with the value specified in ``override``.
-          Typically used in a ``SingleProcessExercise`` or if you want to allow for different solutions other than
-          the one coded up in the solution.
-    """
-
 has_equal_value =  partial(has_expr, test = 'value')
 has_equal_value.__doc__ = """Run targeted student and solution code, and compare returned value.
 
@@ -415,30 +413,52 @@ def has_import(name,
                state=None):
     """Checks whether student imported a package or function correctly.
 
+    Python features many ways to import packages.
+    All of these different methods revolve around the ``import``, ``from`` and ``as`` keywords.
+    ``has_import()`` provides a robust way to check whether a student correctly imported a certain package.
+
+    By default, ``has_import()`` allows for different ways of aliasing the imported package or function.
+    If you want to make sure the correct alias was used to refer to the package or function that was imported,
+    set ``same_as=True``.
+
     Args:
         name (str): the name of the package that has to be checked.
         same_as (bool): if True, the alias of the package or function has to be the same. Defaults to False.
         not_imported_msg (str): feedback message when the package is not imported.
         incorrect_as_msg (str): feedback message if the alias is wrong.
 
-
     :Example:
 
-        Student code::
+        Example 1, where aliases don't matter (defaut): ::
 
-            import numpy as np
-            import pandas as pa
+            # solution
+            import matplotlib.pyplot as plt
 
-        Solution code::
+            # sct
+            Ex().has_import("matplotlib.pyplot")
 
-            import numpy as np
-            import pandas as pd
+            # passing submissions
+            import matplotlib.pyplot as plt
+            from matplotlib import pyplot as plt
+            import matplotlib.pyplot as pltttt
 
-        SCT::
+            # failing submissions
+            import matplotlib as mpl
 
-            Ex().has_import("numpy")  # pass
-            Ex().has_import("pandas") # pass
-            Ex().has_import("pandas", same_as=True) # fail
+        Example 2, where the SCT is coded so aliases do matter: ::
+
+            # solution
+            import matplotlib.pyplot as plt
+
+            # sct
+            Ex().has_import("matplotlib.pyplot", same_as=True)
+
+            # passing submissions
+            import matplotlib.pyplot as plt
+            from matplotlib import pyplot as plt
+
+            # failing submissions
+            import matplotlib.pyplot as pltttt
 
     """
 
@@ -465,9 +485,12 @@ def has_output(text,
                pattern=True,
                no_output_msg=None,
                state=None):
-    """Search student output.
+    """Search student output for a pattern.
 
-    Checks if the output contains a (pattern of) text.
+    Among the student and solution process, the student submission and solution code as a string,
+    the ``Ex()`` state also contains the output that a student generated with his or her submission.
+
+    With ``has_output()``, you can access this output and match it against a regular or fixed expression.
 
     Args:
         text (str): the text that is searched for
@@ -476,15 +499,22 @@ def has_output(text,
 
     :Example:
 
-        SCT::
+        As an example, suppose we want a student to print out a sentence: ::
 
-            Ex().has_output(r'[H|h]i,*\\s+there!')
+            # Print the "This is some ... stuff"
+            print("This is some weird stuff")
 
-        Submissions::
+        The following SCT tests whether the student prints out ``This is some weird stuff``: ::
+
+            # Using exact string matching
+            Ex().has_output("This is some weird stuff", pattern = False)
+
+            # Using a regular expression (more robust)
+            # pattern = True is the default
+            msg = "Print out ``This is some ... stuff`` to the output, " + \\
+                  "fill in ``...`` with a word you like."
+            Ex().has_output(r"This is some \w* stuff", no_output_msg = msg)
 
-            print("Hi, there!")     # pass
-            print("hi  there!")     # pass
-            print("Hello there")    # fail
     """
     rep = Reporter.active_reporter
 
@@ -507,17 +537,14 @@ def has_printout(index,
                  name=None,
                  copy=False,
                  state=None):
-    """Check if the output of print() statement in the solution is in the output the student generated.
+    """Check if the right printouts happened.
+
+    ``has_printout()`` will look for the printout in the solution code that you specified with ``index`` (0 in this case), rerun the ``print()`` call in
+    the solution process, capture its output, and verify whether the output is present in the output of the student.
 
     This is more robust as ``Ex().check_function('print')`` initiated chains as students can use as many
     printouts as they want, as long as they do the correct one somewhere.
 
-    .. note::
-
-        When zooming in on parts of the student submission (with e.g. ``check_for_loop()``), we are not
-        zooming in on the piece of the student output that is related to that piece of the student code.
-        In other words, ``has_printout()`` always considers the entire student output.
-
     Args:
         index (int): index of the ``print()`` call in the solution whose output you want to search for in the student output.
         not_printed_msg (str): if specified, this overrides the default message that is generated when the output
@@ -530,23 +557,63 @@ def has_printout(index,
 
     :Example:
 
-        Solution::
+        Suppose you want somebody to print out 4: ::
 
             print(1, 2, 3, 4)
 
-        SCT::
+        The following SCT would check that: ::
 
             Ex().has_printout(0)
 
-        Each of these submissions will pass::
+        All of the following SCTs would pass: ::
 
             print(1, 2, 3, 4)
             print('1 2 3 4')
             print(1, 2, '3 4')
             print("random"); print(1, 2, 3, 4)
+
+    :Example:
+
+        Watch out: ``has_printout()`` will effectively **rerun** the ``print()`` call in the solution process after the entire solution script was executed.
+        If your solution script updates the value of `x` after executing it, ``has_printout()`` will not work.
+
+        Suppose you have the following solution: ::
+
+            x = 4
+            print(x)
+            x = 6
+
+        The following SCT will not work: ::
+        
+            Ex().has_printout(0)
+
+        Why? When the ``print(x)`` call is executed, the value of ``x`` will be 6, and pythonwhat will look for the output `'6`' in the output the student generated.
+        In cases like these, ``has_printout()`` cannot be used.
+
+    :Example:
+
+        Inside a for loop ``has_printout()`` 
+
+        Suppose you have the following solution: ::
+
+            for i in range(5):
+                print(i)
+
+        The following SCT will not work: ::
+        
+            Ex().check_for_loop().check_body().has_printout(0)
+
+        The reason is that ``has_printout()`` can only be called from the root state. ``Ex()``.
+        If you want to check printouts done in e.g. a for loop, you have to use a `check_function('print')` chain instead: ::
+
+            Ex().check_for_loop().check_body().\\
+                set_context(0).check_function('print').\\
+                check_args(0).has_equal_value()
+
     """
 
-    state.assert_root('has_printout')
+    extra_msg = "If you want to check printouts done in e.g. a for loop, you have to use a `check_function('print')` chain instead."
+    state.assert_root('has_printout', extra_msg=extra_msg)
 
     if not_printed_msg is None:
         not_printed_msg = "Have you used `{{sol_call}}` to do the appropriate printouts?"
@@ -580,12 +647,52 @@ def has_printout(index,
 def has_no_error(incorrect_msg="Have a look at the console: your code contains an error. Fix it and try again!", state=None):
     """Check whether the submission did not generate a runtime error.
 
-    If all SCTs for an exercise pass, before succeeding pythonwhat will automatically check whether
-    the student submission generated an error. If you want to verify whether an error was generated
-    earlier during SCT execution, you can use ``Ex().has_no_error()``.
+    If all SCTs for an exercise pass, before marking the submission as correct pythonwhat will automatically check whether
+    the student submission generated an error. This means it is not needed to use ``has_no_error()`` explicitly.
+
+    However, in some cases, using ``has_no_error()`` explicitly somewhere throughout your SCT execution can be helpful:
+    
+    - If you want to make sure people didn't write typos when writing a long function name.
+    - If you want to first verify whether a function actually runs, before checking whether the arguments were specified correctly.
+    - More generally, if, because of the content, it's instrumental that the script runs without
+      errors before doing any other verifications.
 
     Args:
         incorrect_msg: if specified, this overrides the default message if the student code generated an error.
+
+    :Example:
+
+        Suppose you're verifying an exercise about model training and validation: ::
+
+            # pre exercise code
+            import numpy as np
+            from sklearn.model_selection import train_test_split
+            from sklearn import datasets
+            from sklearn import svm
+
+            iris = datasets.load_iris()
+            iris.data.shape, iris.target.shape
+
+            # solution
+            X_train, X_test, y_train, y_test = train_test_split(
+                iris.data, iris.target, test_size=0.4, random_state=0)
+
+        If you want to make sure that ``train_test_split()`` ran without errors,
+        which would check if the student typed the function without typos and used
+        sensical arguments, you could use the following SCT: ::
+        
+            Ex().has_no_error()
+            Ex().check_function('sklearn.model_selection.train_test_split').multi(
+                check_args(['arrays', 0]).has_equal_value(),
+                check_args(['arrays', 0]).has_equal_value(),
+                check_args(['options', 'test_size']).has_equal_value(),
+                check_args(['options', 'random_state']).has_equal_value()
+            )
+        
+        If, on the other hand, you want to fall back onto pythonwhat's built in behavior,
+        that checks for an error before marking the exercise as correct, you can simply
+        leave of the ``has_no_error()`` step.
+
     """
     state.assert_root('has_no_error')
 
