@@ -4,7 +4,7 @@ from pythonwhat.check_funcs import check_node, check_part, check_part_index, cal
 from pythonwhat.check_logic import multi
 from pythonwhat.has_funcs import has_equal_part_len, has_equal_part, has_equal_value, has_equal_output
 from pythonwhat.check_has_context import has_context
-from functools import partial, update_wrapper
+from functools import partial
 from pythonwhat.Reporter import Reporter
 from pythonwhat.Test import EqualTest, Test
 from pythonwhat import utils
